#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import ecflow_test_util as Test
import unittest 
import shutil   # used to remove directory tree
import os,sys

def test_def_file():
    #return "test.def"
    return "test_tutorial_def_" + str(os.getpid()) + ".def"
    
def test_compile(text):    
    # replace test.def in the text with test_def_file() so that we have a unique file per process
    text = text.replace('test.def',test_def_file())
    
    test_file = "py_u_test_tutorial_" + str(os.getpid()) + ".def"
    file = open(test_file ,'w')
    file.write(text)
    file.close()
    
    # execfile(test_file) only work for python 2.7
    with open(test_file) as f:
        code = compile(f.read(), test_file, 'exec')
        exec(code)  
        
    os.remove(test_file)  

def do_tear_down():
    Ecf.set_debug_equality(False)      
    try: os.remove(test_def_file())
    except: pass
        
######################################################################################################
class TestNewSuite(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)   
           
        home = os.path.join(os.getenv("HOME"),"course")
        self.defs = Defs()
        suite = self.defs.add_suite("test")
        suite.add_variable("ECF_HOME", home)
        suite.add_task("t1")
         
        with Defs() as self.defs2: 
            with self.defs2.add_suite("test") as suite:
                suite.add_variable("ECF_HOME", home)
                suite.add_task("t1") 
         
        with Defs() as self.defs3:  
            self.defs3.add(
                Suite("test",
                    Edit(ECF_HOME=home),
                    Task("t1")))
             
        self.defs4 = Defs() + (Suite("test") + Edit(ECF_HOME=home))
        self.defs4.test += Task("t1")
        
        self.defs5 = Defs().add(
                Suite("test").add(
                    Edit(ECF_HOME=home),
                    Task("t1")))
        
        #print("Creating suite definition")   
        home = os.path.join(os.getenv("HOME"),"course")
        defs = Defs( 
                        Suite('test',
                            Task('t1'),
                            ECF_HOME=home))
        self.assertEqual(self.defs,defs,"defs not equal\n" + str(self.defs) + "\n" + str(defs))

        text = """import os
from ecflow import Defs,Suite,Task,Edit
   
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"),  "course")
defs = Defs( 
        Suite('test',
            Edit(ECF_HOME=home),
            Task('t1')))
#xx print(defs)

defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,defs,"defs not equal\n" + str(test_defs) + "\n" + str(defs))
        
    def test_defs_equal(self):
        self.assertEqual(self.defs, self.defs2, "defs not the equal")
        self.assertEqual(self.defs, self.defs3, "defs not the equal")
        self.assertEqual(self.defs, self.defs4, "defs not the equal")
        self.assertEqual(self.defs, self.defs5, "defs not the equal")

    def tearDown(self):
        do_tear_down()

######################################################################################################
class TestFamilies(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
         
        home = os.path.join(os.getenv("HOME"),  "course") 
        defs = Defs().add(
            Suite("test").add(
                Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                Family("f1").add(
                    Task("t1"),
                    Task("t2"))))
        defs.save_as_defs(test_def_file())
         
        self.defs = defs
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit
         
def create_family_f1():
    return Family("f1",
                Task("t1"),
                Task("t2"))
            
#xx print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"),  "course") 
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os
         
        #print("Creating suite definition")
        home = os.path.join(os.getenv("HOME"),  "course") 
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += Family("f1") + [ Task("t{0}".format(i)) for i in range(1,3) ]
        defs.save_as_defs(test_def_file())

        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))        
 
    def tearDown(self):
        do_tear_down()
 
     
class TestVariables(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
  
        def create_family_f1():
            return Family("f1" ).add(
                Task("t1").add(Edit(SLEEP=20)),
                Task("t2").add(Edit(SLEEP=20)))
  
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add( Suite("test").add(
                            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                            create_family_f1()))
  
        defs.save_as_defs(test_def_file())
 
        self.defs = defs
        
    def test_me0(self):
        text = """#import os
from ecflow import Defs,Suite,Family,Task,Edit

def create_family_f1():
    return Family("f1",
                Task("t1",Edit(SLEEP=20)),
                Task("t2",Edit(SLEEP=20)))
  
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test", 
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))         
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def') 
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me2(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")
  
        with Defs() as defs: 
            defs += Suite("test").add(
                      Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home)) 
            defs.test += Family("f1").add(
                                Task("t1").add(Edit(SLEEP= 20)),
                                Task("t2").add(Edit(SLEEP= 20)))                          
        defs.save_as_defs(test_def_file())
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       

    def test_me3(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")
  
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home)) 
        defs.test += Family("f1") + (Task("t1") + Edit(SLEEP=20)) + (Task("t2") + Edit(SLEEP=20))                     
        defs.save_as_defs(test_def_file())
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       

    def tearDown(self):
        do_tear_down()
         
class TestVariableInheritance(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
         
        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2"))
             
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
                            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                            create_family_f1() ))
        defs.save_as_defs(test_def_file())
   
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit

def create_family_f1():
    return Family("f1",
            Edit(SLEEP=20),
                Task("t1"),
                Task("t2"))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
         
        def create_family_f1():
            return Family("f1") + Edit(SLEEP=20) + Task("t1") + Task("t2")
             
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + create_family_f1()) 
        defs.test += [ Edit(ECF_HOME=home) , Edit(ECF_INCLUDE=home) ]
        defs.save_as_defs(test_def_file())
        
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()
   
         
class TestTriggers(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
         
        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2").add(Trigger("t1 == complete")))
             
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
                            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                            create_family_f1()))
        defs.save_as_defs(test_def_file())
  
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger

def create_family_f1():
    return Family("f1",
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2",Trigger("t1 == complete")))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
         
        home = os.path.join(os.getenv("HOME"), "course")
        with Suite("test") as suite:
            suite += [ Edit(ECF_HOME=home) , Edit(ECF_INCLUDE=home) ]
            suite += Family("f1") + Edit(SLEEP=20) + Task("t1") + Task("t2")
            suite.f1.t2 += Trigger(["t1"]) 
             
        defs = Defs().add( suite )
        defs.save_as_defs(test_def_file())
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()
         
         
class TestEvents(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
 
        def create_family_f1():
            return Family("f1").add(
                        Edit(SLEEP=20),
                        Task("t1"),
                        Task("t2").add(
                            Trigger("t1 == complete"),
                            Event("a"),
                            Event("b")),
                        Task("t3").add(Trigger("t2:a")),
                        Task("t4").add(Trigger("t2:b")))
             
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(
            Suite("test").add(
                Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                create_family_f1() ))
        defs.save_as_defs(test_def_file()) 
  
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Event

def create_family_f1():
    return Family("f1",
                Edit(SLEEP=20),
                Task("t1"),
                Task("t2",
                    Trigger("t1 == complete"),
                    Event("a"),
                    Event("b")),
                Task("t3",
                    Trigger("t2:a")),
                Task("t4",
                    Trigger("t2:b")))
             
#xx print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
 
        def create_family_f1():
            f1 = Family("f1") + [ Edit(SLEEP=20),
                    Task("t1"),
                    Task("t2") + Trigger(["t1"]) + Event("a") + Event("b"),
                    Task("t3") + Trigger("t2:a"),
                    Task("t4") + Trigger("t2:b") ]
            return f1
         
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + Suite("test")
        defs.test += [ Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home), create_family_f1()]
        defs.save_as_defs(test_def_file())   
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()
         
         
class TestComplete(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
 
        def create_family_f1():
            return Family("f1").add(
                Edit(SLEEP= 20),
                Task("t1"),
                Task("t2").add(Trigger("t1 == complete"),
                               Event("a"), Event("b")),
                Task("t3").add(Trigger("t2:a")),
                Task("t4").add(Trigger("t2 == complete"), 
                               Complete("t2:b")  ))
        
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
                Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                create_family_f1() ))
        defs.save_as_defs(test_def_file())   
 
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event

def create_family_f1():
    return Family("f1",
                Edit(SLEEP= 20),
                Task("t1"),
                Task("t2",
                    Trigger("t1 == complete"),
                    Event("a"), 
                    Event("b")),
                Task("t3",
                    Trigger("t2:a")),
                Task("t4",
                    Trigger("t2 == complete"), 
                    Complete("t2:b")))
        
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
 
        def create_family_f1():
            f1 = Family("f1") + Edit(SLEEP=20) 
            f1 += [ Task("t{0}".format(i)) for i in range(1,5) ]
            f1.t2 += [ Trigger(["t1"]), Event("a"), Event("b") ]
            f1.t3 += Trigger("t2:a") 
            f1.t4 += [ Trigger(["t2"]),Complete("t2:b") ]
            return f1
        
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f1()      
        defs.save_as_defs(test_def_file())   
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()

         
class TestMeter(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
 
        def create_family_f1():
            return Family("f1").add(
                        Edit(SLEEP= 20),
                        Task("t1").add(Meter("progress", 1, 100, 90)),
                        Task("t2").add(Trigger("t1 == complete"),
                                       Event("a"),
                                       Event("b")),
                        Task("t3").add(Trigger("t2:a")),
                        Task("t4").add(Trigger("t2 == complete"),
                                       Complete("t2:b")),
                        Task("t5").add(Trigger("t1:progress ge 30")),
                        Task("t6").add(Trigger("t1:progress ge 60")),
                        Task("t7").add(Trigger("t1:progress ge 90")))  
        
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
                Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                create_family_f1() ))
        defs.save_as_defs(test_def_file())   
 
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter

def create_family_f1():
    return Family("f1",
                Edit(SLEEP= 20),
                Task("t1", Meter("progress", 1, 100, 90)),
                Task("t2", Trigger("t1 == complete"), Event("a"), Event("b")),
                Task("t3", Trigger("t2:a")),
                Task("t4", Trigger("t2 == complete"), Complete("t2:b")),
                Task("t5", Trigger("t1:progress ge 30")),
                Task("t6", Trigger("t1:progress ge 60")),
                Task("t7", Trigger("t1:progress ge 90")))  
        
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f1()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
 
        def create_family_f1():
            f1 = Family("f1") + Edit(SLEEP=20) 
            f1 += [ Task("t{0}".format(i)) for i in range(1,8)]
            f1.t1 += Meter("progress", 1, 100, 90) 
            f1.t2 += [ Trigger(["t1"]), Event("a"), Event("b") ]
            f1.t3 += Trigger("t2:a")
            f1.t4 += [ Trigger(["t2"]),Complete("t2:b") ]
            f1.t5 += Trigger("t1:progress ge 30")  
            f1.t6 += Trigger("t1:progress ge 60")  
            f1.t7 += Trigger("t1:progress ge 90")  
            return f1
        
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + (Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f1()   
        defs.save_as_defs(test_def_file())   
         
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()
         
         
class TestTime(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      

        def create_family_f2():
            f2 = Family("f2")
            f2.add_variable("SLEEP", 20)
            f2.add_task("t1").add_time( "00:30 23:30 00:30" ) # start(hh:mm) end(hh:mm) increment(hh:mm)
            f2.add_task("t2").add_day( "sunday" )
   
            # for add_date(): day,month,year; here 0 means every month, and every year
            t3 = f2.add_task("t3")
            t3.add_date(1, 0, 0)           # day month year, first of every month or every year
            t3.add_time( 12, 0 )           # hour, minutes at 12 o'clock
   
            f2.add_task("t4").add_time( 0, 2, True ) # hour, minutes, relative to suite start
                                                     # 2 minutes after family f2 start
            f2.add_task("t5").add_time( 0, 2 )       # hour, minutes suite site
                                                     # 2 minutes past midnight
            return f2
            
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME",    os.path.join(os.getenv("HOME"),  "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"),  "course"))

        #suite.add_family( create_family_f1() )
        suite.add_family( create_family_f2() )
        errors = defs.check()
        assert len(errors) == 0,errors

        defs.save_as_defs(test_def_file())  
 
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date

def create_family_f2():
    return Family("f2",
            Edit(SLEEP=20),
            Task("t1", Time("00:30 23:30 00:30")),     # start(hh:mm) end(hh:mm) increment(hh:mm)
            Task("t2", Day("sunday")),
            Task("t3", Date("1.*.*"), Time("12:00")),  # Date(day,month,year) - * means every day,month,year
            Task("t4", Time("+00:02")),                # + means realative to suite begin/requeue time
            Task("t5", Time("00:02")))                 # 2 minutes past midnight
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            #create_family_f1(),
            create_family_f2()
            ))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
  
        def create_family_f2():
            f1 = Family("f2") + Edit(SLEEP=20)
            f1 += [ Task("t{0}".format(i)) for i in range(1,6) ]
            f1.t1 += Time("00:30 23:30 00:30")  # start(hh:mm) end(hh:mm) increment(hh:mm)
            f1.t2 += Day( "sunday" ) 
            f1.t3 += [ Date("1.*.*"),    
                       Time("12:00")]   
            f1.t4 += Time("+00:02")  
            f1.t5 += Time("00:02")    
            return f1
  
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + ( Suite("test") + Edit(ECF_HOME=home) + Edit(ECF_INCLUDE=home))
        defs.test += create_family_f2()    
        defs.save_as_defs(test_def_file())   
          
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def test_add(self):
        import os
  
        def create_family_f2():
            return Family("f2").add(
                Edit(SLEEP=20),
                Task("t1").add( Time("00:30 23:30 00:30")),  
                Task("t2").add( Day( "sunday" )),
                Task("t3").add( Date("1.*.*"),   
                                Time("12:00")),  
                Task("t4").add( Time("+00:02")), 
                Task("t5").add( Time("00:02")))  
  
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs().add(Suite("test").add(
                Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
                create_family_f2()))
        defs.save_as_defs(test_def_file())   
 
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
        
    def tearDown(self):
        do_tear_down()
     
         
class TestCron(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      

        def create_family_house_keeping():
            cron = Cron()
            cron.set_week_days( [0] )
            cron.set_time_series( "22:30" )
            f2 = Family("house_keeping")
            f2.add_task("clear_log").add_cron(cron)  
            return f2
            
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME",    os.path.join(os.getenv("HOME"),  "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"),  "course"))

        #suite.add_family( create_family_f1() )
        suite.add_family( create_family_house_keeping() )
        errors = defs.check()
        assert len(errors) == 0,errors

        defs.save_as_defs(test_def_file())  
 
        self.defs = defs;
         
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Cron
  
def create_family_house_keeping():
    return Family("house_keeping",
                Task("clear_log",
                    Cron("22:30",days_of_week=[0])))
          
print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_house_keeping()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0,errors

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def tearDown(self):
        do_tear_down()
     
class TestIndentation(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      

        #version = sys.version_info 
        #if version[1] < 7: 
        #    #print "This example requires python version 2.7, but found : " + str(version)
        #    exit(0)

        with Defs() as defs: 
    
            with defs.add_suite("test") as suite:
        
                suite += Edit(ECF_HOME=os.path.join(os.getenv("HOME"),  "course"))
                suite += Edit(ECF_INCLUDE =os.path.join(os.getenv("HOME"),  "course"))
        
            with suite.add_family("f1") as f1:
                f1 += Edit(SLEEP=20)
                f1 += Task("t1", Meter("progress", 1, 100, 90))
                f1 += Task("t2", Trigger("t1 == complete"), Event("a"), Event("b"))
                f1 += Task("t3", Trigger("t2:a"))
                f1 += Task("t4", Trigger("t2 == complete"), Complete("t2:b"))
                f1 += Task("t5", Trigger("t1:progress ge 30"))
                f1 += Task("t6", Trigger("t1:progress ge 60"))
                f1 += Task("t7", Trigger("t1:progress ge 90"))
    
            with suite.add_family("f2") as f2:        
                f2 += Edit(SLEEP=20)
                f2 += Task("t1", Time("00:30 23:30 00:30"))
                f2 += Task("t2", Day("sunday"))
                f2 += Task("t3", Date(1, 0, 0), Time(12, 0))
                f2 += Task("t4", Time(0, 2, True))
                f2 += Task("t5", Time(0, 2))
            
        defs.save_as_defs(test_def_file())
        self.defs = defs;

    def test_preferred(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date

#xx print("Creating suite definition") 
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            Family("f1",
                Edit(SLEEP=20),
                Task("t1", Meter("progress", 1, 100, 90)),
                Task("t2", Trigger("t1 == complete"),Event("a"),Event("b")),
                Task("t3", Trigger("t2:a")),
                Task("t4", Trigger("t2 == complete"), Complete("t2:b")),
                Task("t5", Trigger("t1:progress ge 30")),
                Task("t6", Trigger("t1:progress ge 60")),
                Task("t7", Trigger("t1:progress ge 90"))),
            Family("f2",
                Edit(SLEEP=20),
                Task("t1", Time( "00:30 23:30 00:30" )),
                Task("t2", Day( "sunday" )),
                Task("t3", Date("1.*.*"), Time("12:00")),
                Task("t4", Time("+00:02")),
                Task("t5", Time("00:02")))))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def test_me(self):
        import os
  
        home = os.path.join(os.getenv("HOME"), "course")
        with Defs() as defs:
            with defs.add_suite("test") as suite:
                suite += [ Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home) ]
                with suite.add_family("f1") as f1:
                    f1 += [ Task("t{0}".format(i)) for i in range(1,8)]
                    f1 += Edit(SLEEP=20) 
                    f1.t1 += Meter("progress", 1, 100, 90) 
                    f1.t2 += [ Trigger(["t1"]), Event("a"), Event("b") ]
                    f1.t3 += Trigger("t2:a") 
                    f1.t4 += [ Trigger(["t2"]), Complete("t2:b") ]
                    f1.t5 += Trigger("t1:progress ge 30") 
                    f1.t6 += Trigger("t1:progress ge 60") 
                    f1.t7 += Trigger("t1:progress ge 90") 
                with suite.add_family("f2") as f2:
                    f2 += [ Edit(SLEEP=20),[ Task("t{0}".format(i)) for i in range(1,6)] ]
                    f2.t1 += Time( "00:30 23:30 00:30" )
                    f2.t2 += Day( "sunday" ) 
                    f2.t3 += [ Date("1.*.*"), Time("12:00") ]
                    f2.t4 += Time("+00:02") 
                    f2.t5 += Time("00:02") 
          
        defs.save_as_defs(test_def_file())   
          
        self.assertEqual(self.defs,defs,"defs not equal:\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        do_tear_down()
        
         
class TestLabel(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      

        def create_family_f3():
            fam = Family('f3')
            fam.add_task('t1').add_label("info","")
            return fam
            
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME",    home)
        suite.add_variable("ECF_INCLUDE", home)

        suite.add_family( create_family_f3() )
        errors = defs.check()
        assert len(errors) == 0,errors

        defs.save_as_defs(test_def_file())  
 
        self.defs = defs;
        
    def test_me(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label

def create_family_f3():
    return Family("f3",
                Task("t1",
                Label("info","")))
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f3()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))
        
    def tearDown(self):
        do_tear_down()
         
         
class TestRepeat(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
        
        self.ecf_home     = File.build_dir() + "/libs/pyext/test/data/course_py" + str(sys.version_info[0]) + "_" + str(os.getpid()) # allow paralled runs
        self.ecf_includes = File.source_dir() + "/libs/pyext/test/data/includes"
        #print("self.ecf_home ",self.ecf_home )
 
        try:    os.makedirs( self.ecf_home + "/test/f4/f5")
        except: pass
         
        t1_ecf = '%include <head.h>\n'
        t1_ecf += 'ecflow_client --label=info "My name is %NAME% My value is %VALUE% My date is %DATE%\n'
        t1_ecf += 'ecflow_client --label=date "year:%DATE_YYYY% month:%DATE_MM% day of month:%DATE_DD% day of week:%DATE_DOW%"\n'
        t1_ecf += 'sleep %SLEEP%\n'
        t1_ecf += '%include <tail.h>\n'
         
        self.t1_ecf_path = self.ecf_home + "/test/f4/f5/t1.ecf"
        #print("self.t1_ecf_path",self.t1_ecf_path)
        file = open(self.t1_ecf_path ,"w") 
        file.write(t1_ecf)
        file.close()
    
    def tearDown(self):
        unittest.TestCase.tearDown(self)
        do_tear_down()
        os.remove(self.t1_ecf_path)
        shutil.rmtree(self.ecf_home, ignore_errors=True)  
     
    def test_repeat0(self):
    
        def create_family_f4():
            return Family("f4",
                        Edit(SLEEP=2), 
                        RepeatString("NAME", ["a", "b", "c", "d", "e", "f" ]),
                        Family("f5",
                            RepeatInteger("VALUE", 1, 10),
                            Task("t1",
                                RepeatDate("DATE", 20101230, 20110105),
                                Label("info", ""),
                                Label("date",""))))
              
        defs = Defs( 
                Suite("test",
                    Edit(ECF_HOME=self.ecf_home, ECF_INCLUDE=self.ecf_includes ),
                    create_family_f4()))
        #print(defs)
 
        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)
 
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate
        
def create_family_f4():
    return Family("f4",
                Edit(SLEEP=2), 
                RepeatString("NAME", ["a", "b", "c", "d", "e", "f" ]),
                Family("f5",
                    RepeatInteger("VALUE", 1, 10),
                    Task("t1",
                        RepeatDate("DATE", 20101230, 20110105),
                        Label("info",""),
                        Label("date",""))))
 
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f4()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check() 

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)

    def test_repeat(self):
     
        #print("Creating suite definition") 
        defs = Defs(
                Suite("test",
                    Edit(ECF_HOME=self.ecf_home,ECF_INCLUDE=self.ecf_includes),
                    Family("f4",
                        Edit(SLEEP=2), 
                        RepeatString("NAME", ["a", "b", "c", "d", "e", "f" ]),
                        Family("f5",
                            RepeatInteger("VALUE", 1, 10),
                            Task("t1",
                                RepeatDate("DATE", 20101230, 20110105),
                                Label("info", ""),
                                Label("date",""))))))
        #print(defs)
 
        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)
 
        defs.save_as_defs(test_def_file())

    def test_repeat3(self):
     
        defs = Defs().add( Suite("test") )
        defs.test += [ { "ECF_INCLUDE":self.ecf_includes, "ECF_HOME":self.ecf_home }, 
                        Family("f4") ]
        defs.test.f4 += [   Edit(SLEEP=2),
                            RepeatString("NAME", ["a", "b", "c", "d", "e", "f" ]),
                            Family("f5") ]
        defs.test.f4.f5 += [ RepeatInteger("VALUE", 1, 10),
                            Task("t1")]
        defs.test.f4.f5.t1 += [ RepeatDate("DATE", 20101230, 20110105),
                                Label("info", ""),
                                Label("date","") ]
        #print(defs)
        result = defs.check_job_creation()
        self.assertEqual(result, "", "expected job creation to succeed " + result)
 
        defs.save_as_defs(test_def_file())

        
class TestLimit(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      

        def create_family_f5() :
            f5 = Family("f5")
            f5.add_inlimit("l1")
            f5.add_variable("SLEEP", 20)
            for i in range(1, 10):
                f5.add_task( "t" + str(i) )
            return f5
          
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME",    os.path.join(os.getenv("HOME"),  "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"),  "course"))

        suite.add_limit("l1", 2)
        suite.add_family( create_family_f5() )
        
        defs.save_as_defs(test_def_file())
        
        self.defs = defs   
        
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate,InLimit,Limit
         
def create_family_f5() :
    return Family("f5",
            InLimit("l1"),
            Edit(SLEEP=20),
            [ Task('t{0}'.format(i)) for i in range(1,10) ] )
     
#xx print("Creating suite definition")  
home = os.path.join(os.getenv("HOME"),"course")
defs = Defs(
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            Limit("l1",2),
            create_family_f5()))
#xx print(defs)
 
#xx print("Checking job creation: .ecf -> .job0") 
#print(defs.check_job_creation())
 
#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check()
 
#xx print("Saving definition to file 'test.def'")
defs.save_as_defs(test_def_file()) 
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()


class TestLateAttribute(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
 
        def create_family_f6() :
            f6 = Family("f6")
            f6.add_variable("SLEEP", 120)
            t1 = f6.add_task("t1")
            late = Late()
            late.complete(0,1,True)  # hour,minute,relative,    set late flag if task take longer than a minute
            t1.add_late(late) 
            return f6
       
        defs = Defs()
        suite = defs.add_suite("test")
        suite.add_variable("ECF_HOME",    os.path.join(os.getenv("HOME"),  "course"))
        suite.add_variable("ECF_INCLUDE", os.path.join(os.getenv("HOME"),  "course"))
        suite.add_family( create_family_f6() )
        #print(defs)

        assert len(defs.check()) == 0, defs.check()

        defs.save_as_defs(test_def_file())
        
        self.defs = defs
        
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate,InLimit,Limit,Late
        
def create_family_f6():
    # set late flag if task t1 takes longer than a minute
    return Family("f6",
                Edit(SLEEP=120),
                Task("t1",
                    Late(complete='+00:01')))
     
#xx print("Creating suite definition")   
home = os.path.join(os.getenv("HOME"),"course")
defs = Defs( 
        Suite("test",
            Edit(ECF_HOME=home),Edit(ECF_INCLUDE=home),
            create_family_f6()))
#xx print(defs) 

#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0,defs.check() 

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def tearDown(self):
        do_tear_down()
    

class TestPythonScripting(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)
        
        def create_suite(name) : 
            suite = Suite(name)
            for i in range(1, 7) :
                fam = suite.add_family("f" + str(i))
                for t in ( "a", "b", "c", "d", "e" ) :
                    fam.add_task(t)
            return suite 
    
        self.defs = Defs(create_suite('s1'))   
        
    def test_me(self):
        def create_suite(name) : 
            return Suite(name,
                    [ Family("f{0}".format(i),
                        [ Task(t) for t in ( "a", "b", "c", "d", "e") ]) 
                    for i in range(1,7) ])
     
        defs = Defs(create_suite('s1'))   

        self.assertEqual(self.defs,defs,"defs not equal\n" + str(self.defs) + "\n" + str(defs))       
  
    def tearDown(self):
        Ecf.set_debug_equality(False)      
       

class TestPythonScripting2(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)
              
        def create_sequential_suite(name) :
            suite = Suite(name)
            for i in range(1, 7) :
                fam = suite.add_family("f" + str(i))
                if i != 1: 
                    fam.add_trigger("f" + str(i-1) + " == complete")  # or fam.add_family( "f%d == complete" % (i-1) )
                for t in ( "a", "b", "c", "d", "e" ) :
                    fam.add_task(t) 
            return suite
    
        self.defs = Defs(create_sequential_suite('s1'))   
        
    def test_me(self):
        def create_sequential_suite(name) :
            suite = Suite(name)
            for i in range(1, 7) :
                fam = suite.add_family("f" + str(i))
                if i != 1: 
                    fam += Trigger("f" + str(i-1) + " == complete")  # or fam.add_family( "f%d == complete" % (i-1) )
                for t in ( "a", "b", "c", "d", "e" ) :
                    fam.add_task(t) 
            return suite  
     
        defs = Defs(create_sequential_suite ('s1'))   

        self.assertEqual(self.defs,defs,"defs not equal\n" + str(self.defs) + "\n" + str(defs))       
         
    def tearDown(self):
        Ecf.set_debug_equality(False)      


class TestDataAquistionSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
        
        defs = Defs()
        suite = defs.add_suite("data_aquisition")
        suite.add_repeat( RepeatDay(1) )
        suite.add_variable("ECF_HOME",    os.getenv("HOME") + "/course")
        suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_FILES",   os.getenv("HOME") + "/course/data")
        suite.add_variable("SLEEP","2")
        for city in ( "Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal" ) :
            fcity = suite.add_family(city)
            fcity.add_task("archive")
            for obs_type in ( "observations", "fields", "images" ):
                type_fam = fcity.add_family(obs_type)
                if city in ("Exeter", "Toulouse", "Offenbach"): type_fam.add_time("00:00 23:00 01:00")
                if city in ("Washington") :                     type_fam.add_time("00:00 23:00 03:00")
                if city in ("Tokyo") :                          type_fam.add_time("12:00")
                if city in ("Melbourne") :                      type_fam.add_day( "monday" )
                if city in ("Montreal") :                       type_fam.add_date(1, 0, 0)
         
                type_fam.add_task("get")
                type_fam.add_task("process").add_trigger("get eq complete")
                type_fam.add_task("store").add_trigger("get eq complete")
                    
        #print(defs)
        self.defs = defs
        
    def test_me0(self):
        text = """import os
from ecflow import Defs,Suite,Family,Task,Edit,Trigger,Complete,Event,Meter,Time,Day,Date,Label, \
                   RepeatString,RepeatInteger,RepeatDate

home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
        Suite("data_aquisition",
            RepeatDay(1),
            Edit(ECF_HOME=home),
            Edit(ECF_INCLUDE=home),
            Edit(ECF_FILES=home + "/data"),
            Edit(SLEEP=2)))
for city in ( "Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal" ) :
    fcity = defs.data_aquisition.add_family(city)
    fcity += Task("archive")
    for obs_type in ( "observations", "fields", "images" ):
        type_fam = fcity.add_family(obs_type)
        if city in ("Exeter", "Toulouse", "Offenbach"): type_fam + Time("00:00 23:00 01:00")
        if city in ("Washington") :                     type_fam + Time("00:00 23:00 03:00")
        if city in ("Tokyo") :                          type_fam + Time("12:00")
        if city in ("Melbourne") :                      type_fam + Day( "monday" )
        if city in ("Montreal") :                       type_fam + Date(1, 0, 0)
         
        type_fam + Task("get") + Task("process",Trigger("get eq complete")) + Task("store",Trigger("get eq complete"))         
 
#xx print("Checking job creation: .ecf -> .job0")  
#print(defs.check_job_creation())

#xx print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

#xx print("Saving definition to file 'test.def'")
defs.save_as_defs('test.def')
"""
        test_compile(text)
        test_defs = Defs(test_def_file())
        self.assertEqual(test_defs,self.defs,"defs not equal\n" + str(test_defs) + "\n" + str(self.defs))

    def test_me(self):
        import os
        
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs() + Suite("data_aquisition").add(
                            RepeatDay(1),
                            Edit(ECF_HOME=home),
                            Edit(ECF_INCLUDE=home),
                            Edit(ECF_FILES=home + "/data"),
                            Edit(SLEEP=2))
        for city in ( "Exeter", "Toulouse", "Offenbach", "Washington", "Tokyo", "Melbourne", "Montreal" ) :
            fcity = defs.data_aquisition.add_family(city)
            fcity += Task("archive")
            for obs_type in ( "observations", "fields", "images" ):
                type_fam = fcity.add_family(obs_type)
                if city in ("Exeter", "Toulouse", "Offenbach"): type_fam + Time("00:00 23:00 01:00")
                if city in ("Washington") :                     type_fam + Time("00:00 23:00 03:00")
                if city in ("Tokyo") :                          type_fam + Time("12:00")
                if city in ("Melbourne") :                      type_fam + Day( "monday" )
                if city in ("Montreal") :                       type_fam + Date(1, 0, 0)
         
                type_fam += [ Task("get"),Task("process"),Task("store") ]
                type_fam.process += Trigger("get eq complete") 
                type_fam.store += Trigger("get eq complete")     

        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)      


class TestOperationalSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
   
        defs = Defs()
        suite = defs.add_suite("operation_suite")
        suite.add_repeat( RepeatDay(1) )
        suite.add_variable("ECF_HOME",    os.getenv("HOME") + "/course")
        suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
        suite.add_variable("ECF_FILES",   os.getenv("HOME") + "/course/oper")
  
        # Defines the triggers for the first cycle
        cycle_triggers = "1"
        for cycle in ( "00" , "12" ):

            if cycle == "12" :
                last_step = 240
            else:
                last_step = 24
 
            fcycle_fam = suite.add_family(cycle)
            fcycle_fam.add_variable("CYCLE", cycle)
            fcycle_fam.add_variable("LAST_STEP", last_step)
      
            if cycle_triggers != "1" :
                fcycle_fam.add_trigger(cycle_triggers)
                 
            analysis_fam = fcycle_fam.add_family("analysis")
            analysis_fam.add_task("get_observations")
            analysis_fam.add_task("run_analysis").add_trigger("get_observations == complete")
            analysis_fam.add_task("post_processing").add_trigger("run_analysis == complete")
 
            forecast_fam = fcycle_fam.add_family("forecast")
            forecast_fam.add_trigger("analysis == complete")
            forecast_fam.add_task("get_input_data") 
            run_forecast_task = forecast_fam.add_task("run_forecast") 
            run_forecast_task.add_trigger("get_input_data == complete")
            run_forecast_task.add_meter("step", 0, last_step, last_step)
 
            archive_fam = fcycle_fam.add_family("archive")
            fam_analsis = archive_fam.add_family("analysis")
            fam_analsis.add_variable("TYPE","analysis")
            fam_analsis.add_variable("STEP","0")
            fam_analsis.add_trigger("../analysis/run_analysis == complete")
            fam_analsis.add_task("save")
    
            for i in range(6, last_step+1, 6):  
                step_fam = fam_analsis.add_family("step_" + str(i))     
                step_fam.add_variable("TYPE", "forecast")  
                step_fam.add_variable("STEP", i)  
                step_fam.add_trigger("../../forecast/run_forecast:step ge " + str(i))
                step_fam.add_task("save")

            # Defines the triggers for the next cycle
            cycle_triggers = "./" + cycle + " == complete"   
                 
        #print(defs)
        self.defs = defs
        
    def test_sol1(self):
        import os
         
        home = os.getenv("HOME") + "/course"
        cycle_triggers = None
        last_step = { "12": 240,
                      "00": 24, }
         
        def cycle_trigger(cycle):
            if cycle == "12": return Trigger("./00 == complete")
            return None
         
        defs = Defs(
            Suite("operation_suite",
                RepeatDay(1),
                Edit(ECF_HOME= home),
                Edit(ECF_INCLUDE= home),
                Edit(ECF_FILES= home + "/oper"),
                [ Family(cycle,
                    Edit(CYCLE=cycle),
                    Edit(LAST_STEP=last_step[cycle]),
        
                    cycle_trigger(cycle),
                    
                    Family("analysis",
                        Task("get_observations"),
                        Task("run_analysis", Trigger(["get_observations"])),
                        Task("post_processing", Trigger(["run_analysis"]))
                    ),
    
                    Family("forecast",
                        Trigger(["analysis"]),
                        Task("get_input_data"),
                        Task("run_forecast",
                            Trigger(["get_input_data"]),
                            Meter("step", 0, last_step[cycle])),
                    ),
                                           
                    Family("archive",
                        Family("analysis",
                            Edit(TYPE="analysis"),
                            Edit(STEP=0),
                            Trigger("../analysis/run_analysis == complete"),
                            Task("save"),
                            [ Family("step_{0}".format(i),
                                Edit(TYPE="forecast"),
                                Edit(STEP=i),
                                Trigger("../../forecast/run_forecast:step ge {0}".format(i)),
                                Task("save"))
                             for i in range(6, last_step[cycle]+1, 6) ]
                        )
                    )
                ) for cycle in ( "00" , "12" ) ] 
            )
        )
        #print(defs)
        
        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)      


class TestBackArchivingSolution(unittest.TestCase):
    def setUp(self):
        Ecf.set_debug_equality(True)      
    
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs()
        suite = defs.add_suite("back_archiving")
        suite.add_repeat( RepeatDay(1) )
        suite.add_variable("ECF_HOME",    home)
        suite.add_variable("ECF_INCLUDE", home)
        suite.add_variable("ECF_FILES",   home + "/back")
        suite.add_variable("SLEEP", "2")
        suite.add_limit("access", 2)
        for kind in ( "analysis", "forecast", "climatology", "observations", "images" ):
            find_fam = suite.add_family(kind)
            find_fam.add_repeat( RepeatDate("DATE", 19900101, 19950712) )
            find_fam.add_variable("KIND", kind)
            find_fam.add_task("get_old").add_inlimit("access")
            find_fam.add_task("convert").add_trigger("get_old == complete")
            find_fam.add_task("save_new").add_trigger("convert == complete")
             
        #print(defs)
         
        self.defs = defs
         
    def test_sol1(self):
        import os
        home = os.path.join(os.getenv("HOME"), "course")
        defs = Defs( 
            Suite("back_archiving",
                RepeatDay(1),
                Edit(ECF_HOME=home),
                Edit(ECF_INCLUDE=home),
                Edit(ECF_FILES= home + "/back"),
                Edit(SLEEP=2),
                Limit("access", 2),
                [ Family(kind,
                    RepeatDate( "DATE", 19900101, 19950712 ),
                    Edit(KIND=kind),
                    Task("get_old",  InLimit("access")),
                    Task("convert",  Trigger("get_old == complete")),
                    Task("save_new", Trigger("convert == complete")))
                  for kind in ( "analysis", "forecast", "climatology", "observations", "images" ) ] ))
        #print(defs)

        self.assertEqual(self.defs, defs, "defs not equal")

    def tearDown(self):
        Ecf.set_debug_equality(False)      

if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



import os
from ecflow import (
    Defs,
    Suite,
    Family,
    Task,
    Edit,
    Trigger,
    Complete,
    Event,
    Meter,
    Time,
    Day,
    Date,
    Label,
    RepeatString,
    RepeatInteger,
    RepeatDate,
    InLimit,
    Limit,
)


def create_family_f5():
    return Family(
        "f5",
        InLimit("l1"),
        Edit(
            SLEEP=20,
            HOST="?????",
            ECF_OUT="/tmp/%s" % os.getenv("USER"),
            ECF_LOGHOST="%HOST%",
            ECF_LOGPORT="?????",  # port=$((35000 + $(id -u))) run this on the command line
            ECF_JOB_CMD="ssh %HOST% 'mkdir -p %ECF_OUT%/%SUITE%/%FAMILY%; %ECF_JOB% > %ECF_JOBOUT% 2>&1 &'",
        ),
        [Task("t{}".format(i)) for i in range(1, 10)],
    )


print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
    Suite(
        "test",
        Edit(ECF_INCLUDE=home, ECF_HOME=home),
        Limit("l1", 2),
        create_family_f5(),
    )
)
print(defs)

print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Checking trigger expressions")
assert len(defs.check()) == 0, defs.check()

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

#  code for testing derivation works
import os
import sys
import ecflow
import ecflow_test_util as Test

class MyDefs(ecflow.Defs): pass
class MySuite(ecflow.Suite): pass
class MyFamily(ecflow.Family): pass
class MyTask(ecflow.Task): pass
class MyClient(ecflow.Client): pass

if sys.version_info[0] > 2:
    class MySuite2(ecflow.Suite):
        def __init__(self, name):
            super().__init__(name)
    
    class MyFamily2(ecflow.Family):
        def __init__(self, name):
            super().__init__(name)
    
    class MyTask2(ecflow.Task):
        def __init__(self, name):
            super().__init__(name)

    n = MySuite2('s')
    print(n)

    n = MyFamily2('f')
    print(n)

    n = MyTask2('t')
    print(n)

else:
    print("Python2 ignoring derivation test")


if __name__ == "__main__":
    
    Test.print_test_start(os.path.basename(__file__))
    print("All tests pass")



#!/usr/bin/env python2.7
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

import sys
sys.path.append("../../../admin/2.0")
import rest
import argparse

PARSER = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
PARSER.add_argument('--file', help="Tar file to upload")
ARGS = PARSER.parse_args()
    
conflunce_base_url = "https://software-test.ecmwf.int/wiki"
confluence = rest.Confluence(conflunce_base_url,"deploy","deploy2013")
page_id = confluence.get_page_id("ECFLOW","Releases") 
if page_id is None:
    sys.exit(1)

attachment = confluence.get_attachment_id(page_id,ARGS.file)
if attachment is not None:
    print("space key:",space_key," allready has an attachment for file ",ARGS.file)
else:
    confluence.create_attachment(page_id,"%TARBALL_COMMENT%",ARGS.file)
    
#     # ===========================================================================
#     CL = ecflow.Client("localhost", 4141)
#     try:
#         CL.ping() 
#         print "ping worked";
#         
#         alter = 'alter add variable X "X XX" /ecflow/test_local_actions; alter add variable xx "y a" /ecflow/test_local_actions;'
#         CL.group(alter)
#  
#     except RuntimeError, ex:
#         print "Error: " + str(ex)



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# SCRATCH test for ecflow python api

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate,RepeatDateList, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys
import os

class Test_get_attr(unittest.TestCase):
    def test_get_attr(self):
        defs = Defs() + (Suite('s') + Family('f').add(Task('t') + Edit(var="1")))
        defs += Edit(var="1")
        
        self.assertIsInstance(defs.s, Suite, "Expected suite")
        self.assertIsInstance(defs.s.f, Family, "Expected family")
        self.assertIsInstance(defs.s.f.t, Task, "Expected Task but found " + str(type(defs.s.f.t)))
        
        self.assertIsInstance(defs.var, Variable, "Expected Variable but found " + str(type(defs.var)))
        self.assertIsInstance(defs.s.f.t.var, Variable, "Expected Variable but found " + str(type(defs.s.f.t.var)))
        
    def test_get_attr_generated_variables(self):
        defs = Defs() + (Suite('s') + Family('f').add((Task('t')  + Edit(var="1") + RepeatDate("YMD", 20100111, 20100115, 2)),
                                                      (Task('t2') + Edit(var="1") + RepeatDateList("YMD",[20100111, 20100115 ]))))
        defs.s.f.t += Meter("meter",0,100)
        defs.s.f.t += Event("event")
        defs.s.f.t += Limit("limitx",10)
        #PrintStyle.set_style(Style.STATE)
        #print(defs)
        
        self.assertTrue(defs.ECF_MICRO, "expected generated variable")
        self.assertTrue(defs.ECF_HOME, "expected generated variable")
        self.assertTrue(defs.ECF_JOB_CMD , "expected generated variable")
        self.assertTrue(defs.ECF_KILL_CMD , "expected generated variable")
        self.assertTrue(defs.ECF_STATUS_CMD , "expected generated variable")
        self.assertTrue(defs.ECF_URL_CMD , "expected generated variable")
        self.assertTrue(defs.ECF_LOG  , "expected generated variable")
        self.assertTrue(defs.ECF_INTERVAL   , "expected generated variable")
        self.assertTrue(defs.ECF_LISTS   , "expected generated variable")
        self.assertTrue(defs.ECF_CHECK   , "expected generated variable")
        self.assertTrue(defs.ECF_CHECKOLD   , "expected generated variable")
        self.assertTrue(defs.ECF_CHECKINTERVAL   , "expected generated variable")
        self.assertTrue(defs.ECF_CHECKMODE   , "expected generated variable")
        self.assertTrue(defs.ECF_TRIES   , "expected generated variable")
        self.assertTrue(defs.ECF_VERSION   , "expected generated variable")
        self.assertTrue(defs.ECF_PORT   , "expected generated variable")
        self.assertTrue(defs.ECF_HOST    , "expected generated variable")

        self.assertTrue(defs.s.SUITE, "expected generated variable")
        self.assertEqual(defs.s.SUITE.value() , 's', "expected suite name of 's' but found")
        self.assertTrue(defs.s.ECF_DATE , "expected generated variable")
        self.assertTrue(defs.s.YYYY , "expected generated variable")
        self.assertTrue(defs.s.DOW , "expected generated variable")
        self.assertTrue(defs.s.DOY , "expected generated variable")
        self.assertTrue(defs.s.DATE , "expected generated variable")
        self.assertTrue(defs.s.DAY , "expected generated variable")
        self.assertTrue(defs.s.DD , "expected generated variable")
        self.assertTrue(defs.s.MM , "expected generated variable")
        self.assertTrue(defs.s.MONTH , "expected generated variable")
        self.assertTrue(defs.s.ECF_CLOCK , "expected generated variable")
        self.assertTrue(defs.s.ECF_TIME , "expected generated variable")
        self.assertTrue(defs.s.TIME  , "expected generated variable")
    
        self.assertTrue(defs.s.f.FAMILY   , "expected generated variable")
        self.assertTrue(defs.s.f.FAMILY1   , "expected generated variable")

        self.assertTrue(defs.s.f.t.TASK    , "expected generated variable")
        self.assertEqual(defs.s.f.t.TASK.value() , 't', "expected task name of 's'")
        self.assertTrue(defs.s.f.t.ECF_JOB     , "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_SCRIPT      , "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_JOBOUT      , "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_TRYNO      , "expected generated variable")
        self.assertEqual(defs.s.f.t.ECF_TRYNO.value() , '0', "expected task try no of '0'")
        self.assertTrue(defs.s.f.t.ECF_RID      , "expected generated variable")
        self.assertTrue(defs.s.f.t.ECF_NAME      , "expected generated variable")
        self.assertEqual(defs.s.f.t.ECF_NAME.value()  , '/s/f/t', "expected task ECF_NAME of '/s/f/t'")
        self.assertTrue(defs.s.f.t.ECF_PASS       , "expected generated variable")

        self.assertEqual(defs.s.f.t.YMD.value() , '20100111', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_YYYY.value() , '2010', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_MM.value() , '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_DD.value() , '11', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_DOW.value() , '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.YMD_JULIAN.value() , '2455208', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t.event.value() , 0, "expected generated event of value 0 but found " + str(defs.s.f.t.event.value()))
        self.assertEqual(defs.s.f.t.meter.value() , 0, "expected generated meter of value 0 but found " + str(defs.s.f.t.meter.value()))
        self.assertEqual(defs.s.f.t.limitx.value() , 0, "expected generated limit of value 0 but found " + str(defs.s.f.t.limitx.value()))#
        
        self.assertEqual(defs.s.f.t2.YMD.value() , '20100111', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_YYYY.value() , '2010', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_MM.value() , '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_DD.value() , '11', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_DOW.value() , '1', "expected generated YMD of value")
        self.assertEqual(defs.s.f.t2.YMD_JULIAN.value() , '2455208', "expected generated YMD of value")


if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



import os
from ecflow import (
    Defs,
    Suite,
    Family,
    Task,
    Edit,
    Trigger,
    Complete,
    Event,
    Meter,
    Cron,
)


def create_family_house_keeping():
    return Family("house_keeping", Task("clear_log", Cron("22:30", days_of_week=[0])))


print("Creating suite definition")
home = os.path.join(os.getenv("HOME"), "course")
defs = Defs(
    Suite("test", Edit(ECF_INCLUDE=home, ECF_HOME=home), create_family_house_keeping())
)
print(defs)

print("Checking job creation: .ecf -> .job0")
print(defs.check_job_creation())

print("Checking trigger expressions")
errors = defs.check()
assert len(errors) == 0, errors

print("Saving definition to file 'test.def'")
defs.save_as_defs("test.def")



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# This test checks trigger evaluation uses date arithmetic when
# the expression reference a repeat DATE variable.
# Addition and subtraction for repeat date, should follow data arithmetic
# The repeat variable in the expression must be on the LHS(left hand side)
# of the expression, otherwise integer arithmetic is used
import os
import ecflow
import ecflow_test_util as Test


def test_repeat_arithmetic(repeat_to_add,repeat_to_add2):
    defs = ecflow.Defs()
    s1 = defs.add_suite("s1");
    t1 = s1.add_task("t1").add_repeat( repeat_to_add );
    t2 = s1.add_task("t2").add_trigger("t1:YMD ge 20100601");
 
    # Check trigger expressions
    assert len(defs.check()) == 0,  "Expected no errors in parsing expressions."
 
    # Initial value of repeat is 20090101 hence trigger should fail to evaluate
    assert t2.evaluate_trigger() == False, "Expected trigger to evaluate. 20090101 >= 20100601"
   
    # Check end of month - 1
    t2.change_trigger("t1:YMD - 1 eq 20081231")
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090101 - 1  == 20081231"

    # check addition
    t2.change_trigger("t1:YMD + 1 eq 20090102");    
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090101 + 1  == 20090102"

    # Check the end of each month + 1
    t1.delete_repeat();
    t1.add_repeat( repeat_to_add2 ) 
    t2.change_trigger("t1:YMD + 1 eq 20090201");   
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090131 + 1  == 20090201"


def test_repeat_datetime_arithmetic(repeat_to_add,repeat_to_add2):

    defs = ecflow.Defs()
    s1 = defs.add_suite("s1")
    t1 = s1.add_task("t1").add_repeat( repeat_to_add )
    t2 = s1.add_task("t2").add_trigger("t1:DT ge 20100601T000000")

    # Check trigger expressions
    print(f"## defs: {defs}")
    assert len(defs.check()) == 0,  "Expected no errors in parsing expressions."

    # Initial value of repeat is 20090101 hence trigger should fail to evaluate
    assert t2.evaluate_trigger() == False, "Expected trigger to evaluate. 20090101T000000 >= 20100601T000000"

    # Check end of month - 1 day
    t2.change_trigger("t1:DT - 86400 eq 20081231T000000")
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090101T000000 - 86400  == 20081231T000000"

    # check addition
    t2.change_trigger("t1:DT + 86400 eq 20090102T000000");
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090101T000000 + 86400  == 20090102T000000"

    # Check the end of each month + 1 day
    t1.delete_repeat();
    t1.add_repeat( repeat_to_add2 )
    t2.change_trigger("t1:DT + 86400 eq 20090201T000000");
    assert t2.evaluate_trigger(), "Expected trigger to evaluate. 20090131 + 86400  == 20090201T000000"


if __name__ == "__main__":
    Test.print_test_start(os.path.basename(__file__))

    test_repeat_arithmetic(ecflow.RepeatDate("YMD", 20090101, 20091231, 1),
                           ecflow.RepeatDate("YMD", 20090131, 20101231, 1))
    test_repeat_arithmetic(ecflow.RepeatDateList("YMD", [20090101, 20091231]),
                           ecflow.RepeatDateList("YMD", [20090131, 20101231]))

    test_repeat_datetime_arithmetic(ecflow.RepeatDateTime("DT", "20090101T000000", "20091231T000000", "24:00:00"),
                                    ecflow.RepeatDateTime("DT", "20090131T000000", "20101231T000000", "24:00:00"))

    print("All Tests pass")



#!/usr/bin/env python3

import re
import pathlib
import subprocess

cmd_type_terms = {"child": "child command", "user": "user command", "option": ""}


def run_command(cmd):
    print(f"*** Executing: {cmd}")
    p = subprocess.run(cmd, capture_output=True, text=True)
    return p.stdout, p.stderr


class Entry:
    def __init__(self, name, type, desc):
        self.name = name
        self.type = type
        self.desc = desc

    def __repr__(self):
        return f"Entry('{self.name}', '{self.type}', '{self.desc}')"


def load_help_table(command):
    entries = []
    o, _ = run_command(command)
    o = o.split("\n")
    for line in o:
        m = re.search(r"""(\S+)\s+(child|user|option)\s+(.*)""", line)
        if m and m.groups and len(m.groups()) == 3:
            entry = Entry(m.group(1), m.group(2), m.group(3))
            entries.append(entry)
    return entries


def load_commands():
    return load_help_table(["ecflow_client", "--help=summary"])


def load_options():
    return load_help_table(["ecflow_client", "--help=option"])


def render_single_page_rst(name):
    o, _ = run_command(["ecflow_client", f"--help={name}"])

    # prefix each line of the output with the necessary indentation
    o = '\n'.join(['   ' + line for line in o.split('\n')]) + '\n'

    title = f"{name}"
    txt = f"""
.. _{name}_cli:

{title}
{"/" * len(title)}

::

"""

    txt += o
    return txt


def render_user_option_rst():

    title = f"user"
    txt = f"""
.. _user_cli:

{title}
{"/" * len(title)}

::

   
   user
   ----
   
   Specifies the user name used to contact the server. Must be used in combination with option --password.
   
   
"""
    return txt


def render_index_rst():
    title = "Command line interface (CLI)"
    txt = f"""
.. _ecflow_cli:

{title}
{"/" * len(title)} 

The command line interface is provided by the :term:`ecflow_client`
executable. Note that most of the commands that you
execute using :ref:`ecflow_ui` are actually CLI commands. 

The very first argument to :term:`ecflow_client` specifies the command and must be prefixed with ``--``, e.g. ``--load`` in the example below:

.. code-block:: shell

    ecflow_client --load=host1.3141.check

The comprehensive :ref:`list of ecflow_client commands <ecflow_client_commands>` is presented below.
These commands can be combined with :ref:`ecflow_client common options <ecflow_client_options>` to further customise the
:term:`ecflow_client` behaviour.

The list of commands, amongst other details, can be displayed by using the option ``--help``.

.. code-block:: shell

    ecflow_client --help


Some pages about CLI specific details can be found here:

.. toctree::
    :maxdepth: 1
    
    desc/index.rst
"""

    txt += """

.. list-table:: List of :term:`ecflow_client` commands
    :header-rows: 1
    :width: 100%
    :widths: 20 20 60
    :name: ecflow_client_commands

    * - Command
      - Type
      - Description
    """

    command_entries = load_commands()
    for entry in command_entries:
        txt += f"""
    * - :ref:`{entry.name}_cli` 
      - :term:`{cmd_type_terms[entry.type]}`
      - {entry.desc}
"""

    txt += f"""

.. list-table:: List of common options for `ecflow_client` commands
    :header-rows: 1
    :width: 100%
    :widths: 20 80
    :name: ecflow_client_options

    * - Option
      - Description
"""

    option_entries = load_options()
    for entry in option_entries:
        txt += f"""
    * - :ref:`{entry.name}_cli`
      - {entry.desc}
"""

    txt += f"""

.. toctree::
    :maxdepth: 1
    :hidden:
    
"""

    for entry in command_entries:
        txt += f"""    {entry.name} <api/{entry.name}.rst>\n"""
    for entry in option_entries:
        txt += f"""    {entry.name} (option) <api/{entry.name}.rst>\n"""

    return txt


if __name__ == "__main__":

    # Render and store index.rst
    content = render_index_rst()
    with open("index.rst", "w") as f:
        f.write(content)

    # Ensure api sub-folders is present
    pathlib.Path("api").mkdir(parents=True, exist_ok=True)

    # Render and store each of the command/option.rst
    entries = load_commands()
    for entry in entries:
        content = render_single_page_rst(entry.name)
        with open(f"api/{entry.name}.rst", "w") as f:
            f.write(content)

    entries = load_options()
    for entry in entries:
        if entry.name == "user":
            content = render_user_option_rst()
        else:
            content = render_single_page_rst(entry.name)
        with open(f"api/{entry.name}.rst", "w") as f:
            f.write(content)



# Copyright 2009-2019 ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import sys

import ecflow


class Indentor:
    """This class manages indentation, for use with context manager
    It is used to correctly indent the definition node tree hierarchy
    """

    _index = 0

    def __init__(self):
        Indentor._index += 1

    def __del__(self):
        Indentor._index -= 1

    @classmethod
    def indent(cls, the_file):
        for i in range(Indentor._index):
            the_file.write(" ")


class DefsTraverser:
    """Traverse the ecflow.Defs definition and write to file.

    This demonstrates that all nodes in the node tree and all attributes are accessible.
    Additionally the state data is also accessible. This class will write state data as
    comments. If the definition was returned from the server, it allows access to latest
    snapshot of the state data held in the server.
    """

    def __init__(self, defs):
        assert isinstance(defs, ecflow.Defs), "Expected ecflow.Defs as first argument"
        self.__defs = defs

    def write_to_file(self, file_name):
        assert isinstance(
            file_name, str
        ), "Expected a string argument. Representing a file name"
        self.__file = open(file_name, "w")
        for extern in self.__defs.externs:
            self.__writeln("extern " + extern)
        for suite in self.__defs.suites:
            self.__write("suite ")
            self.__print_node(suite)
            clock = suite.get_clock()
            if clock:
                indent = Indentor()
                self.__writeln(str(clock))
                del indent
            self.__print_nc(suite)
            self.__writeln("endsuite")
        self.__file.close()

    def __print_nc(self, node_container):
        indent = Indentor()
        for node in node_container.nodes:
            if isinstance(node, ecflow.Task):
                self.__write("task ")
                self.__print_node(node)
            else:
                self.__write("family ")
                self.__print_node(node)
                self.__print_nc(node)
                self.__writeln("endfamily")
        del indent

    def __print_node(self, node):
        self.__file.write(node.name() + " # state: " + str(node.get_state()) + "\n")

        indent = Indentor()
        defStatus = node.get_defstatus()
        if defStatus != ecflow.DState.queued:
            self.__writeln("defstatus " + str(defStatus))

        autocancel = node.get_autocancel()
        if autocancel:
            self.__writeln(str(autocancel))

        repeat = node.get_repeat()
        if not repeat.empty():
            self.__writeln(str(repeat) + " # value: " + str(repeat.value()))

        late = node.get_late()
        if late:
            self.__writeln(str(late) + " # is_late: " + str(late.is_late()))

        complete_expr = node.get_complete()
        if complete_expr:
            for part_expr in complete_expr.parts:
                trig = "complete "
                if part_expr.and_expr():
                    trig = trig + "-a "
                if part_expr.or_expr():
                    trig = trig + "-o "
                self.__write(trig)
                self.__file.write(part_expr.get_expression() + "\n")
        trigger_expr = node.get_trigger()
        if trigger_expr:
            for part_expr in trigger_expr.parts:
                trig = "trigger "
                if part_expr.and_expr():
                    trig = trig + "-a "
                if part_expr.or_expr():
                    trig = trig + "-o "
                self.__write(trig)
                self.__file.write(part_expr.get_expression() + "\n")

        for var in node.variables:
            self.__writeln("edit " + var.name() + " '" + var.value() + "'")
        for meter in node.meters:
            self.__writeln(str(meter) + " # value: " + str(meter.value()))
        for event in node.events:
            self.__writeln(str(event) + " # value: " + str(event.value()))
        for label in node.labels:
            self.__writeln(str(label) + " # value: " + label.value())
        for limit in node.limits:
            self.__writeln(str(limit) + " # value: " + str(limit.value()))
        for inlimit in node.inlimits:
            self.__writeln(str(inlimit))
        for the_time in node.times:
            self.__writeln(str(the_time))
        for today in node.todays:
            self.__writeln(str(today))
        for date in node.dates:
            self.__writeln(str(date))
        for day in node.days:
            self.__writeln(str(day))
        for cron in node.crons:
            self.__writeln(str(cron))
        for verify in node.verifies:
            self.__writeln(str(verify))
        for zombie in node.zombies:
            self.__writeln(str(zombie))

        del indent

    def __write(self, the_string):
        Indentor.indent(self.__file)
        self.__file.write(the_string)

    def __writeln(self, the_string):
        Indentor.indent(self.__file)
        self.__file.write(the_string + "\n")


try:
    # Create the client. This will read the default environment variables
    ci = ecflow.Client("localhost", "4143")

    # Get the node tree suite definition as stored in the server
    # The definition is retrieved and stored on the variable 'ci'
    ci.sync_local()

    # access the definition retrieved from the server
    server_defs = ci.get_defs()

    if server_defs is None:
        print("The server has no definition")
        sys.exit(1)

    # Traverse server definition writing all state as comments.
    traverser = DefsTraverser(server_defs)
    traverser.write_to_file("server.defs")

except RuntimeError as e:
    print("failed: " + str(e))



#pyflow Method
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

#pyflow is a pure python module on top of ecflow native API ;
# it is proposed to facilitate suites definitions. 
# It is not yet provided in default standard location. 
# It can be used, adding this one line to the definition script.

import sys; sys.path.append("/home/ma/emos/def/pyflow")

# Enter the following python code into a file i.e. test.py :

#!/usr/bin/env python2.7
import os
from pyflow import (Suite, Family, Task, Variable)
   
print("Creating suite definition")   
with Suite("test" ) as suite: 
 ECF_HOME = os.path.join(os.getenv("HOME"),  "course") # python vriable
 Variable("ECF_HOME", ECF_HOME)  # suite variable
 Task("t1")
print(suite.ecflow_definition()) 

#Then run as a python script:


#pyflow
#Using pyflow, header files are generated and placed in the expected location (ECF_INCLUDE) when the suite is deployed:
suite.deploy_suite()



# (C) Copyright 2011- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

"""
Generate .cmake file to set source-file specific compiler flags based on
rules defined in a JSON file.
"""

from argparse import ArgumentParser
from fnmatch import fnmatch
import logging
from json import JSONDecoder
from os import path

log = logging.getLogger('gen_source_flags')


def match(source, pattern, op, flags, indent=0):
    if fnmatch(source, pattern):

        suff = '' if op[0] in ('+', '=', '/') else ' (nested pattern)'
        log.debug('%s-> pattern "%s" matches "%s"%s',
                  ' ' * (indent + 1), pattern, source, suff)

        if op[0] == "+":
            flags += [flag for flag in op[1:] if flag not in flags]
            log.debug('%sappending %s --> flags: %s', ' ' * (indent + 2), op[1:], flags)

        elif op[0] == "=":
            flags = op[1:]
            log.debug('%ssetting %s --> flags: %s', ' ' * (indent + 2), op[1:], flags)

        elif op[0] == "/":
            flags = [flag for flag in flags if flag not in op[1:]]
            log.debug('%sremoving %s --> flags: %s', ' ' * (indent + 2), op[1:], flags)

        else:  # Nested rule
            log.debug('%sapplying nested rules for "%s" (flags: %s)',
                      ' ' * (indent + 2), pattern, flags)
            for nested_pattern, nested_op in op:
                flags = match(source, nested_pattern, nested_op, flags, indent + 2)

    return flags


def generate(rules, out, default_flags, sources, debug=False):
    logging.basicConfig(level=logging.DEBUG if debug else logging.INFO,
                        format='-- %(levelname)s - %(name)s: %(message)s')

    with open(path.expanduser(rules)) as f:
        rules = JSONDecoder(object_pairs_hook=list).decode(f.read())

    with open(path.expanduser(out), 'w') as f:
        for source in sources:
            log.debug('%s (default flags: "%s")', source, default_flags)
            flags = default_flags.split()
            for pattern, op in rules:
                flags = match(source, pattern, op, flags)

            if flags:
                log.debug(' ==> setting flags for %s to %s', source, ' '.join(flags))
                f.write('set_source_files_properties(%s PROPERTIES COMPILE_FLAGS "%s")\n'
                        % (source, ' '.join(flags)))
            else:
                log.debug(' ==> flags for %s empty', source)


def main():
    """Parse arguments"""
    parser = ArgumentParser(description=__doc__)
    parser.add_argument('rules', metavar='RULES.json', help='JSON rules file')
    parser.add_argument('out', metavar='OUT.cmake', help='CMake script to generate')
    parser.add_argument('default_flags', help='Default compiler flags to use')
    parser.add_argument('sources', metavar='file', nargs='+', help='Path to file to apply rules to')
    parser.add_argument('--debug', '-d', action='store_true', help='Log debug messages')
    generate(**vars(parser.parse_args()))

if __name__ == '__main__':
    main()



#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# SCRATCH test for ecflow python api

from ecflow import Alias, AttrType, Autocancel, CheckPt, ChildCmdType, Client, Clock, Cron, DState, Date, Day, Days, \
                  Defs, Ecf, Event, Expression, Family, FamilyVec, File, Flag, FlagType, FlagTypeVec, InLimit, \
                  JobCreationCtrl, Label, Late, Limit, Meter, Node, NodeContainer, NodeVec, PartExpression, PrintStyle, \
                  Repeat, RepeatDate, RepeatDay, RepeatEnumerated, RepeatInteger, RepeatString, SState, State, Style, \
                  Submittable, Suite, SuiteVec, Task, TaskVec, Time, TimeSeries, TimeSlot, Today, UrlCmd, Variable, \
                  VariableList, Verify, WhyCmd, ZombieAttr, ZombieType, ZombieUserActionType, Trigger, Complete, Edit, Defstatus
import unittest 
import sys
import os

def test_compile(text):
    test_file = "py_u_test_tutorial.def"
    file = open(test_file ,'w')
    file.write(text)
    file.close()
    execfile(test_file)
    os.remove(test_file)  

class TestCron(unittest.TestCase):
    def setUp(self):
        pass
    
    def test_pass(self):
        pass

    def test_fail(self):
        pass
        
if __name__ == "__main__":
    unittest.main()
    print("All Tests pass")



#!/usr/bin/env python
from distutils.core import setup, Extension
import os
import sys
import glob

# ========================================================================
# Usage:
#   cd $WK; cd ecbuild/release/; sh -x ../../cmake.sh release; make ; make install
#   calling 'sh -x ../../cmake.sh release' will generate setup.py from setup.py.in
# Issues:
#   Cannot test cmake install, since it will always install to 
#     /usr/local/apps/python/current/lib/python2.7/site-packages/ecflow
#
# To test install manually, we can:
#   python3 setup.py --help-commands # help
#
#   cd $WK/libs/pyext
#   rm -rf build/                     # for a clean build
#   python setup.py build_ext         # build C/C++ extensions (compile/link to build directory)
#   python setup.py install --home=~
#
# See: http://docs.python.org/2/distutils/apiref.html?highlight=extension#distutils.core.Extension
#
# ==========================================================================
# Permissions: The permission of configured file are wrong: 2 choices
# o Make sure origin file, has the right permissions
# o Copy file to different directory, and change the permissions
#   since file(COPY) does rename files
#   configure_file(setup.py.in /tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/CMakeFiles/setup.py)
#       now copy the temporary into the final destination, setting the permissions
#   file(COPY /tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/CMakeFiles/setup.py
#     DESTINATION /tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox
#     FILE_PERMISSIONS OWNER_READ OWNER_WRITE OWNER_EXECUTE GROUP_READ
#     GROUP_EXECUTE WORLD_READ WORLD_EXECUTE)
# ============================================================================
        
#=============================================================================
# define the directories to search for include files
# to get this to work, you will need to include the path
# to your boost installation and  ecflow includes
boost_root=os.getenv("BOOST_ROOT") 
include_dirs = [ "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../3rdparty/cereal/include",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/core/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/attribute/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/node/parser/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/node/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/base/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/base/src/cts",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/base/src/stc",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/simulator/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/../libs/client/src",
                 "/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/src",
                 boost_root,
               ]

# define the library directories to include any extra libraries that may be needed.
# Give preference to release libs   
boost_lib_dir = boost_root + "/stage/lib/"
library_dirs = ['/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/core',
                '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/attribute/',
                '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/node/',
                '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/base/',
                '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/simulator/',
                '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow-build-sandbox/libs/client/',
                boost_lib_dir,
               ]

# define the libraries to link with this includes the boost lib
libraries = [ 'core' , 'nodeattr', 'node', 'base', 'libsimu', 'libclient',
              'boost_system',
              'boost_filesystem',
              'boost_program_options',
              'boost_date_time', 
              'boost_python36' ]

# Using gcc-4.8 with boost 1.53 linux requires these flags
extra_compile_args = ['-ftemplate-depth-512','-Wno-unused-variable','-Wno-deprecated-declarations','-Wno-unused-local-typedefs','-Wno-maybe-uninitialized' ]
extra_link_args = []

# create the extension and add it to the python distribution
# o glob.glob(os.path.join('src', '*.cpp'))
#   This expand the list of cpp files that need to be compiled
#
# The configuation below installs to:
#   o /usr/local/apps/python/current/lib/python2.7/site-packages/ecflow
#     when, "python setup.py install" is used:
#   lib/python2.7/site-packages/ecflow/ecflow.so
#                                     __init__.py               
setup( name='ecflow', 
       version='5.13.8', 
       author      = 'ECMWF',
       description = """ecflow Python interface""",
       packages = [ 'ecflow' ],
       url = 'https://confluence.ecmwf.int/display/ECFLOW/ecflow+home',
       license = 'Apache',
       package_dir={'ecflow': '/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/ecflow'},
       ext_modules=[ Extension( 
                              'ecflow.ecflow', 
                              glob.glob(os.path.join('/tmp/ecflow-packaging-stage.2ArLog4mxh/ecflow/libs/pyext/src', '*.cpp')), 
                              include_dirs=include_dirs, 
                              library_dirs=library_dirs, 
                              libraries=libraries,
                              extra_compile_args=extra_compile_args,
                              extra_link_args=extra_link_args
                              )
                    ],
   )



#!/usr/bin/env python2.7
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
# Name        :
# Author      : Avi
# Revision    : $Revision: #10 $
#
# Copyright 2009- ECMWF.
# This software is licensed under the terms of the Apache Licence version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.
#////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8

# =============================================================================
# Code for testing *any* definition
#   Since any ad hoc definition will reference local directories in the
#   ECF_ variables, we need to remove them and inject our own.
#
#   This test was created aid the performance testing of job generation
#   It is tied to ANode/parser/test/TestJobGenPerf.cpp
# =============================================================================
import ecflow
import os       # for getenv
import sys
import shutil   # used to remove directory tree
import argparse # for argument parsing     

def delete_variables_affecting_job_generation(node): 
    """delete customer related ECF variables, these will point to directories
       that don't exist. Its ok we will regenerate our own local ones"""
    var = node.find_variable("ECF_HOME")
    if not var.empty() :  
        node.delete_variable("ECF_HOME")    
    var = node.find_variable("ECF_FILES")
    if not var.empty() :  
        node.delete_variable("ECF_FILES")    
    var = node.find_variable("ECF_INCLUDE")
    if not var.empty() :  
        node.delete_variable("ECF_INCLUDE")    
    var = node.find_variable("ECF_JOB_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_JOB_CMD")    
    var = node.find_variable("ECF_KILL_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_KILL_CMD")    
    var = node.find_variable("ECF_STATUS_CMD")
    if not var.empty() :  
        node.delete_variable("ECF_STATUS_CMD")    
    var = node.find_variable("ECF_OUT")
    if not var.empty() :  
        node.delete_variable("ECF_OUT")    
      
def traverse_container(node_container):
    """Recursively traverse definition node hierarchy and delete
       the variables that affect job generation.
    """
    delete_variables_affecting_job_generation(node_container)
    for node in node_container.nodes:
        delete_variables_affecting_job_generation(node)
        if not isinstance(node, ecflow.Task):
            traverse_container(node)  
    
if __name__ == "__main__":
    
    DESC = """Will allow any definition to be loaded and played on the server
            This is done by:
            o Remove existing ECF_ variables that affect job generation. 
              i.e variables that refer to customer specific directories are removed
            o Allows ECF_HOME to specified, defaults to ./CUSTOMER/ECF_HOME
            o Generates the scripts(.ecf files) automatically based on the definition.
              i.e if a task has events,meters,labels then the client request for these are
              automatically injected in the generated .ecf script files
            o Will clear out existing data both on disk and on the server to allow 
              multiple re-runs of this script. ** If this is an issue please use
              a test server **
            This programs assumes that ecflow module is accessible.
            """    
    PARSER = argparse.ArgumentParser(description=DESC,  
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    PARSER.add_argument('defs_file', 
                        help="The definition file")
    PARSER.add_argument('--ecf_home', default="/var/tmp/ma0/ECFLOW_TEST/TestJobGenPerf/ECF_HOME",
                        help="Directory to be used for generated scripts(ECF_HOME), defaults to /var/tmp/ma0/ECFLOW_TEST/TestJobGenPerf/ECF_HOME")
    PARSER.add_argument('--verbose', nargs='?', default=False, const=True, type=bool,
                        help="Show verbose output")
    ARGS = PARSER.parse_args()
    ARGS.defs_file = os.path.expandvars(ARGS.defs_file) # expand references to any environment variables
    print(ARGS)    
    
    # If running on local work space, use /libs/pyext/test/data/CUSTOMER/ECF_HOME as ecf_home
    if not ARGS.ecf_home:
        if os.getenv("WK") is None:
            print("No ecf_home specified. Please specify a writable directory")
            sys.exit(1)
        ARGS.ecf_home = "/var/tmp/ma0/ECFLOW_TEST/TestJobGenPerf/ECF_HOME"
        if ARGS.verbose:
            print("Workspace is defined") 
            print("using /Client/bin/gcc\-4.5/debug/ecflow_client")

    print("Using ECF_HOME=" + ARGS.ecf_home)
    print("removing directory " + ARGS.ecf_home)
    try:
        shutil.rmtree(ARGS.ecf_home)   
    except:
        pass   
        
    if not os.path.exists(ARGS.ecf_home): 
        print(ARGS.ecf_home + " directory does not exist, creating")
        os.makedirs(ARGS.ecf_home)

    print("\nloading the definition from the input arguments(" + ARGS.defs_file + ")\n")
    
    try:
        DEFS = ecflow.Defs(ARGS.defs_file)
    except RuntimeError as ex:
        print("   ecflow.Defs(" + ARGS.defs_file + ") failed:\n" + str(ex))
        sys.exit(1)
    
    if ARGS.verbose: 
        print("remove test data associated with the DEFS, so we start fresh, Allows rerun")
    for suite in DEFS.suites:
        dir_to_remove = ARGS.ecf_home + suite.get_abs_node_path()
        if ARGS.verbose: 
            print("   Deleting directory: " + dir_to_remove + "\n")
        shutil.rmtree(dir_to_remove, True)  
        
    if ARGS.verbose: 
        print("remove remote reference to ECF_HOME and ECF_INCLUDE, since we inject or own\n")
    for suite in DEFS.suites:
        print("add variables required for script generation, for all suites\n")
        traverse_container(suite)
  
        suite.add_variable("ECF_HOME", ARGS.ecf_home)
        suite.add_variable("ECF_INCLUDE", ARGS.ecf_home + "/includes")
        suite.add_variable("ECF_CLIENT_EXE_PATH", os.getenv("WK") + "/Client/bin/gcc\-4.8/debug/ecflow_client")
        suite.add_variable("SLEEP", "10")  # not strictly required since default is 1 second


    
    # ecflow.PrintStyle.set_style(ecflow.Style.STATE)
    #if ARGS.verbose: 
    #print DEFS
    DEFS.generate_scripts()

    if ARGS.verbose: 
        print("\nchecking script file generation, pre-processing & variable substitution\n")
    JOB_CTRL = ecflow.JobCreationCtrl()
    DEFS.check_job_creation(JOB_CTRL)       
    assert len(JOB_CTRL.get_error_msg()) == 0, JOB_CTRL.get_error_msg()
    
    
    newDefs = ARGS.ecf_home + "/../" + os.path.basename(ARGS.defs_file)
    print("Saving modified defs as " + newDefs) 
    DEFS.save_as_defs(newDefs)



import os
import ecflow

defs = ecflow.Defs()
suite = defs.add_suite("back_archiving")
suite.add_repeat(ecflow.RepeatDay(1))
suite.add_variable("ECF_HOME", os.getenv("HOME") + "/course")
suite.add_variable("ECF_INCLUDE", os.getenv("HOME") + "/course")
suite.add_variable("ECF_FILES", os.getenv("HOME") + "/course/back")
suite.add_variable("SLEEP", "2")
suite.add_limit("access", 2)
for kind in ("analysis", "forecast", "climatology", "observations", "images"):
    find_fam = suite.add_family(kind)
    find_fam.add_repeat(ecflow.RepeatDate("DATE", 19900101, 19950712))
    find_fam.add_variable("KIND", kind)
    find_fam.add_task("get_old").add_inlimit("access")
    find_fam.add_task("convert").add_trigger("get_old == complete")
    find_fam.add_task("save_new").add_trigger("convert == complete")



#!/usr/bin/env python
""" one module to store needed function to translate SMS-ECFLOW suites """
import sys
ECF_MODE = "ecflow" # "sms"

# RAW_MODE:
# False: translate *_CMD (submit, kill, status) variables values for %ECF_ + number args
# True:  just translate the name, and %ECF_, not arguments number

RAW_MODE = False    
# ECF_MODE = "sms"
CDP = "/usr/local/apps/sms/bin/cdp"
subm = "/home/ma/emos/bin/trimurti"
kill = subm
stat = subm

DICT_SMS_ECF = {
    'SMSNAME': 'ECF_NAME',
    'SMSNODE': 'ECF_HOST',
    'SMSPASS': 'ECF_PASS',
    'SMS_PROG': 'ECF_PORT',
    'SMSFILES': 'ECF_FILES',
    'SMSINCLUDE': 'ECF_INCLUDE',
    'SMSTRIES':   'ECF_TRIES',
    'SMSTRYNO': 'ECF_TRYNO',
    'SMSTRIE': 'ECF_TRIE',
    'SMSHOME': 'ECF_HOME',
    'SMSRID': 'ECF_RID',
    'SMSJOB': 'ECF_JOB',
    'SMSJOBOUT': 'ECF_JOBOUT',
    'SMSOUT': 'ECF_OUT',
    'SMSCHECKOLD': 'ECF_CHECKOLD',
    'SMSCHECK': 'ECF_CHECK',
    'SMSLOG': 'ECF_LOG',
    'SMSLIST': 'ECF_LIST',
    'SMSPASSWD': 'ECF_PASSWD',
    'SMSSERVER': 'ECF_SERVER',
    'SMSMICRO': 'ECF_MICRO',
    'SMSPID': 'ECF_PID',
    'SMSHOST': 'ECF_HOST',
    'SMSDATE': 'ECF_DATE',
    'SMSURL': 'ECF_URL',
    'SMSURLBASE': 'ECF_URLBASE',
    'SMSCMD': 'ECF_JOB_CMD',
    'SMSKILL': 'ECF_KILL_CMD',
    'SMSSTATUSCMD': 'ECF_STATUS_CMD',
    'SMSJOBCHECK': 'ECF_CHECK_CMD',
    'SMSURLCMD': 'ECF_URL_CMD',
    'SMSWEBACCES': 'ECF_WEBACCES',
    'SMS_VER': 'ECF_VER',
    'SMS_VERSION': 'ECF_VERSION',
    'SMSLOGHOST': 'ECF_LOGHOST',
    'SMSLOGPORT': 'ECF_LOGPORT',
    'SMSLOGTIMEOUT': 'ECF_LOGTIMEOUT',
}

DICT_ECF_SMS = dict((v, k) for k, v in DICT_SMS_ECF.items())
      
def translate(name, value=None):
    """ find and replace submit, kill and status command, according to
    sms/ecflow mode """
    value2 = value
    transl = name
    smssubmit = "trimurti"
    ecfsubmit = "/home/ma/emos/bin/ecflow_submit"

    def add_ext(name, value):
            if "_v2" in value: rc = " %ECF_JOB% %SCHOST% %USER%"
            elif "kill" in value or "stat" in value:
                if "WSHOST" in value: rc = RIDCLWS
                else: rc = RIDCL
            # if "submit" in value: 
            #     if "WSHOST" in value: rc = JOBCLWS
            #     else: rc = JOBCL
            elif "WSHOST" in value: rc = JOBCLWS
            else: rc = JOBCL

            if name in  ('ECF_JOB_CMD', 'SMSCMD'):
                 rc = SUBCMD + " " + rc 
            elif name in ('ECF_KILL_CMD', 'SMSKILL'):
                if "submit" in KILLCMD:
                    rc = KILLCMD + " %s kill" % rc
                else: rc = KILLCMD + " %s " % rc
            elif name in ('ECF_STATUS_CMD', 'SMSSTATUS', 
                           'ECF_CHECK_CMD', 'SMSJOBCHECK'):
                if "submit" in STATCMD:
                     rc = STATCMD + " %s status" % rc
                else: rc = STATCMD + " %s" % rc
            return rc

    if ECF_MODE == "ecflow" and name in DICT_SMS_ECF:
        if name in ('SMSCMD', 'SMSKILL', 'SMSSTATUSCMD', 'SMSHOME'):
            value2 = value.replace('SMS', 'ECF_')
            value  = value2.replace(smssubmit, ecfsubmit)
        elif name in ('ECF_JOB_CMD', 'ECF_KILL_CMD', 'ECF_STATUS_CMD'):
            value2 = value.replace('SMS', 'ECF_')
            value  = value2.replace(smssubmit, ecfsubmit)
        transl = DICT_SMS_ECF[name]

    elif ECF_MODE == "sms"  and name in DICT_ECF_SMS:
        if name in  ('ECF_CMD_CMD', 'ECF_KILL_CMD', 'ECF_STATUS_CMD'):
            value2 = value.replace('ECF_', 'SMS')  
        transl = DICT_ECF_SMS[name]

    else: pass

    if ECF_MODE == "sms" and RAW_MODE:
        value2 = value.replace("ECF_", "SMS")
        return transl, value2
 
    elif ECF_MODE == "sms" and "CMD" in name and not "submit" in value:
        if name in ('ECF_JOB_CMD'): name = "SMSCMD"
        elif name in ('ECF_KILL_CMD'): name = "SMSKILL"
        elif name in ('ECF_STATUS_CMD'): name = "SMSSTATUS"
        value2 = value.replace("ECF_", "SMS")
        value  = value2

    elif ECF_MODE == "sms":
             SUBCMD  = subm
             KILLCMD = kill
             STATCMD = stat

             JOBCL  = ' %USER% %SCHOST% %SMSJOB% %SMSJOBOUT%'
             RIDCL  = ' %USER% %SCHOST% %SMSRID% %SMSJOB% %SMSJOBOUT%'
             STATL  = ' %USER% %SCHOST% %SMSRID% %SMSJOB%'
             STATLWS  = ' %USER% %WSHOST% %SMSRID% %SMSJOB%'

             JOBCLWS = JOBCL.replace("%SC", "%WS")
             RIDCLWS = RIDCL.replace("%SC", "%WS") # .replace("HOST%", "HOST% %SMSRID%")
             if name in  ('ECF_JOB_CMD', 'SMSCMD'):
                 value2 = add_ext(name, value)
             elif name in ('ECF_KILL_CMD', 'SMSKILL'):
                 if "submit" in KILLCMD:
                     value2 = KILLCMD + add_ext(value, value) + " kill"
                 else: value2 = KILLCMD + add_ext(value, value)
             elif name in ('ECF_STATUS_CMD', 'SMSSTATUS', 
                           'ECF_CHECK_CMD', 'SMSJOBCHECK'):
                 if "submit" in STATCMD:
                     value2 = STATCMD + add_ext(value, value) + " status"
                 else: value2 = STATCMD + add_ext(value, value)
             return transl, value2
    return transl, value


def get(name, host="localhost", port=314159):
    import os, pwd
    os.getlogin = lambda: pwd.getpwuid(os.getuid())[0]
    user = os.getlogin()
    cmd = "setenv -i HOME USER; set SMS_PROG %d; login %s %s 1; status > /dev/null ; get; show > %s; " % (port, host, user, name)
    os.system("cdp -c '%s'" % cmd)

def ignore(): pass

ignored = ( "action", "owner", "text", "migrate", "automigrate", "autorestore")
def sms2ecf(orig, dest):
   import re
   with open(dest, 'w') as fop:
       with open(orig, 'r') as source:
            try:
                for line in source:
                    for key in ignored:
                        if " %s " % key in line: 
                            continue                
                    print(line, file=fop)
            except:
                print("oops")

def load(name, host="localhost", port=31415):
    import ecflow as ec
    client = ec.Client(host, port)
    try:
        client.load(name)
    except:
        client.replace("/", name)

if __name__ == "__main__":
    import sys
    try:
        sms  = sys.argv[1]
        prog = sys.argv[2]
        ecf  = sys.argv[3]
        port = sys.argv[4]
    except:
        sms  = "localhost"
        prog = 314159
        ecf  = "localhost"
        port = 31415

    before = "from_sms.exp"
    become = "prep_ecf.exp"
    get(name=before)
    sms2ecf(orig=before, dest=become)
    load(name=become)
